@@ -11,6 +11,6 @@ pub mod decode;
 pub use values::*;
 pub use config::StrConfig;
 pub use buffer::Buffer;
-pub use encode::{encode_proto, encode_field, encode_number, encode_map, encode_type, encode_sure_type, 
+pub use encode::{encode_proto, encode_field, encode_number, encode_varint, encode_map, encode_type, encode_sure_type, 
                  encode_str_raw};
-pub use decode::{decode_proto, decode_field, decode_number, decode_map, decode_str_raw};
+pub use decode::{decode_proto, decode_field, decode_number, decode_varint, decode_map, decode_str_raw};
